/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery;

import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;

import java.util.Map;


/**
 * This interface defines the contract for a data source for the site. Implementations of this interface should create
 * and return a site map consisting of {@link NodeMatcher} nodes that describe the site map. The returns map should have
 * each unique {@link NodeMatcher} listed as the key and a {@link java.util.List} of {@link NodeMatcher} nodes that
 * describe the children of that key.
 * <p/>
 * <b>Implementation notes: </b>The site discover data source will be used to read in (from whatever source) the site
 * information and create a map of nodes to child nodes as the site map. Each of these nodes will be of {@link
 * NodeMatcher} type to allow the component to determine if a node matches any given node in the site map.
 * <p/>
 * <b>Thread Safety:</b> Implementations of this interface should be thread safe.
 *
 * @author Pops, TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public interface SiteDiscovery {
    /**
     * This method returns a map of {@link NodeMatcher} keys with a {@link java.util.List} of {@link NodeMatcher}
     * elements as the value. The key {@link NodeMatcher} represents the parent and the value (the list) represents the
     * child values. The value can be <tt>null</tt> or an empty list to represent no child elements (i.e. it's leaf
     * node).
     *
     * @return A non-<tt>null</tt> map of parent {@link NodeMatcher} to a {@link java.util.List} of {@link NodeMatcher}
     *         children
     *
     * @throws BreadCrumbException if an exception occurs retrieving the site map
     */
    public Map getSiteMap() throws BreadCrumbException;
}


