/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;


/**
 * This class contains all test cases for {@link AbstractNodeDiscovery}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class AbstractNodeDiscoveryTest extends BaseBreadCrumbTrailTagTest {
    /**
     * This is a value used as valid title argument in testcases.
     */
    private static final String TITLE = "__title__";
    /**
     * This is a value used as valid url argument in testcases.
     */
    private static final String URL = "u://rl";
    /**
     * This is an empty string for checking empty String argument behavior.
     */
    private static final String EMPTY_STRING = "  ";
    /**
     * This is the currentNode determined by the dummy implementation.
     */
    private BreadCrumbNode currentNode;
    /**
     * This is the instance to be tested, created in {@link #setUp()}.
     */
    private AbstractNodeDiscoveryTest.DummyNodeDiscovery nodeDiscovery;

    /**
     * This method tests the correct behavior of {@link AbstractNodeDiscovery#AbstractNodeDiscovery()}.
     */
    public void testCtor() {
        new DummyNodeDiscovery();
    }

    /**
     * This method tests the correct behavior of {@link AbstractNodeDiscovery#getTitle()} and {@link
     * AbstractNodeDiscovery#setTitle(String)} .
     */
    public void testGetSetTitle() {
        assertEquals(null, nodeDiscovery.getTitle());
        nodeDiscovery.setTitle(TITLE);
        assertEquals(TITLE, nodeDiscovery.getTitle());
        nodeDiscovery.setTitle(null);
        assertEquals(null, nodeDiscovery.getTitle());
        nodeDiscovery.setTitle(EMPTY_STRING);
        assertEquals(null, nodeDiscovery.getTitle());
    }

    /**
     * This method tests the correct behavior of {@link AbstractNodeDiscovery#doStartTag()}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testDoStartTag() throws Exception {
        pageContext.removeAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE);
        assertEquals(TagSupport.SKIP_BODY, nodeDiscovery.doStartTag());
        assertEquals(currentNode, pageContext.getAttribute(BreadCrumbTrailTag.BREADCRUMB_CURRENT_NODE));
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        currentNode = new BreadCrumbNode(AbstractNodeDiscoveryTest.URL, AbstractNodeDiscoveryTest.TITLE);
        nodeDiscovery = new DummyNodeDiscovery();
        nodeDiscovery.setPageContext(pageContext);
    }

    /**
     * This is a mock AbstractNodeDiscovery subclass used in testing the AbstractNodeDiscovery functionality.
     */
    private class DummyNodeDiscovery extends AbstractNodeDiscovery {

        /**
         * This method is called to retrieve the node representation of the current page. If the current page could not
         * be found or is not applicable - simply return <tt>null</tt>. The returned node should include any query
         * strings available.
         *
         * @param pageContext a non-<tt>null</tt> page context to use
         *
         * @return a {@link BreadCrumbNode} representing the current url or <tt>null</tt> if the node couldn't be
         *         determined
         *
         * @throws IllegalArgumentException if the page context is <tt>null</tt>
         * @throws BreadCrumbException      if an exception occurs discovering the page
         */
        public BreadCrumbNode getCurrentNode(final PageContext pageContext) throws BreadCrumbException {
            return currentNode;
        }
    }
}
