/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.UnknownNamespaceException;
import com.topcoder.web.ui.tag.breadcrumbtrail.pathdiscovery.DynamicPathDiscovery;
import org.apache.cactus.JspTestCase;

import java.io.InputStream;
import java.io.InputStreamReader;


/**
 * This class is the base class for all unit test classes for the Bread Crumb Trail Tag component, that depend on
 * properly configured environment. It performs the setup necessary for the Bread Crumb Trail Tag component to work
 * properly.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public abstract class BaseBreadCrumbTrailTagTest extends JspTestCase {
    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} namespace of the {@link BreadCrumbTrailTag} class.
     */
    protected static final String BREAD_CRUMB_TRAIL_TAG_NAMESPACE = "com.topcoder.web.ui.tag.breadcrumbtrail";
    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} namespace of {@link #ObjectFactory} used by the {@link
     * BreadCrumbTrailTag} class.
     */
    protected static final String BREAD_CRUMB_TRAIL_TAG_OBJECT_FACTORY_NAMESPACE = BreadCrumbTrailTag.class.getName();
    /**
     * This is the {@link com.topcoder.util.config.ConfigManager} namespace of {@link #ObjectFactory} used by the {@link
     * DynamicPathDiscovery} class.
     */
    protected static final String DYNAMIC_PATH_DISCOVERY_OBJECT_FACTORY_NAMESPACE = DynamicPathDiscovery.class
        .getName();

    /**
     * This is the filename of the config file containing the default configuration used in the test cases.
     */
    protected static final String DEFAULT_BREAD_CRUMB_TRAIL_TAG_CONFIGURATION = "defaultConfig.xml";

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        loadBreadCrumbTrailTagConfiguration(DEFAULT_BREAD_CRUMB_TRAIL_TAG_CONFIGURATION);
    }

    /**
     * This method does the cleanup after test run.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        clearConfig();
    }

    /**
     * This method removes all Bread Crumb Trail Tag-specific configuration namespaces plus the namespaces of dependant
     * components from the ConfigManager singleton of this class' classloader and then loads the configuration with the
     * given filename/classloader path into the ConfigManager instance.
     *
     * @param filename the config file to be loaded as new Bread Crumb Trail Tag configuration
     *
     * @throws Exception                in case some unexpected Exception occurs
     * @throws IllegalArgumentException in case the given filename is <tt>null</tt>
     */
    protected void loadBreadCrumbTrailTagConfiguration(final String filename) throws Exception {
        if (filename == null) {
            throw new IllegalArgumentException("The parameter named [filename] was null");
        }

        // clear config manager data
        clearConfig();

        // load the configuration for component,
        ConfigManager.getInstance().add(filename);
    }

    /**
     * This method removes all namespaces that are used by the Time Tracker Report component in testcases from the
     * ConfigManager singleton instance of this class' classloader.
     */
    protected void clearConfig() {
        removeNamespace(BREAD_CRUMB_TRAIL_TAG_NAMESPACE);
        removeNamespace(BREAD_CRUMB_TRAIL_TAG_OBJECT_FACTORY_NAMESPACE);
        removeNamespace(DYNAMIC_PATH_DISCOVERY_OBJECT_FACTORY_NAMESPACE);
    }

    /**
     * This method removes the given namespace from the ConfigManager singleton instance of this class' classloader.
     *
     * @param namespace the namespace to be removed from the ConfigManager, must not be <tt>null</tt>
     *
     * @throws NullPointerException in case the given namespace was <tt>null</tt>
     */
    protected void removeNamespace(final String namespace) {
        if (namespace == null) {
            throw new NullPointerException("The parameter named [namespace] was null");
        }

        final ConfigManager cm = ConfigManager.getInstance();
        if (cm.existsNamespace(namespace)) {
            try {
                cm.removeNamespace(namespace);
            } catch (UnknownNamespaceException ignored) {
                // This may happen if the config namespace to be removed did not exist
                // (which only can occur because of threading issues, as we check for
                // namespace existence before removing), but as the result is the
                // desired ConfigManager state (the namespace not existing any more
                // in CM), we can ignore this
            }
        }
    }

    /**
     * This method is used to load resource file available in the classpath into a String.
     *
     * @param filename the name of the file to be loaded using the {@link ClassLoader} of this class
     *
     * @return the content of the file
     *
     * @throws Exception in case an I/O error occurs, e.g. the file cannot be found
     */
    protected String readResourceIntoString(final String filename) throws Exception {
        final InputStream stream = getClass().getClassLoader().getResourceAsStream(filename);
        final StringBuffer expectedBuffer = new StringBuffer();
        final InputStreamReader inputStreamReader = new InputStreamReader(stream, "UTF-8");
        try {
            final char[] buffer = new char[4096];
            int bytesRead;
            while ((bytesRead = inputStreamReader.read(buffer)) != -1) {
                expectedBuffer.append(buffer, 0, bytesRead);
            }
        } finally {
            if (stream != null) {
                stream.close();
            }
        }
        return expectedBuffer.toString();
    }
}
