/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.web.ui.tag.breadcrumbtrail.nodeformatter;

import com.topcoder.web.ui.tag.breadcrumbtrail.BaseBreadCrumbTrailTagTest;
import com.topcoder.web.ui.tag.breadcrumbtrail.BreadCrumbException;


/**
 * This class contains all test cases for {@link NodeFormatterUtils}.
 *
 * @author TCSDEVELOPER
 * @version 2.0
 * @since 2.0
 */
public class NodeFormatterUtilsTest extends BaseBreadCrumbTrailTagTest {
    /**
     * This is a value used as valid title argument in testcases.
     */
    private static final String TITLE = "__title__";
    /**
     * This is a value used as valid url argument in testcases.
     */
    private static final String URL = "u://rl";

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getOptionalPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetOptionalPageContextAttribute() throws Exception {
        assertEquals(null, NodeFormatterUtils.getOptionalPageContextAttribute(pageContext, TITLE, String.class));
        pageContext.setAttribute(TITLE, URL);
        assertEquals(URL, NodeFormatterUtils.getOptionalPageContextAttribute(pageContext, TITLE, String.class));
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getOptionalPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     * <p/>
     * <b>Failure scenario:</b>given pageContext is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetOptionalPageContextAttributeNullPageContext() throws Exception {
        try {
            NodeFormatterUtils.getOptionalPageContextAttribute(null, TITLE, String.class);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getOptionalPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     * <p/>
     * <b>Failure scenario:</b>given attributeName is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetOptionalPageContextAttributeNullAttributeName() throws Exception {
        try {
            NodeFormatterUtils.getOptionalPageContextAttribute(pageContext, null, String.class);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getOptionalPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     * <p/>
     * <b>Failure scenario:</b>given attributeName is an empty String
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetOptionalPageContextAttributeEmptyAttributeName() throws Exception {
        try {
            NodeFormatterUtils.getOptionalPageContextAttribute(pageContext, " ", String.class);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getOptionalPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     * <p/>
     * <b>Failure scenario:</b>given expectedType is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetOptionalPageContextAttributeNullExcpectedType() throws Exception {
        try {
            NodeFormatterUtils.getOptionalPageContextAttribute(pageContext, TITLE, null);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getOptionalPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     * <p/>
     * <b>Failure scenario:</b>retrieved value has wrong type
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetOptionalPageContextAttributeWrongType() throws Exception {
        pageContext.setAttribute(TITLE, new Integer(6));
        try {
            NodeFormatterUtils.getOptionalPageContextAttribute(pageContext, TITLE, String.class);
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getRequiredPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetRequiredPageContextAttribute() throws Exception {
        pageContext.setAttribute(TITLE, URL);
        assertEquals(URL, NodeFormatterUtils.getRequiredPageContextAttribute(pageContext, TITLE, String.class));
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getRequiredPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     * <p/>
     * <b>Failure scenario:</b>value retrieved from context was <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetRequiredPageContextAttributeNullValueRetrieved() throws Exception {
        try {
            NodeFormatterUtils.getRequiredPageContextAttribute(pageContext, TITLE, String.class);
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getRequiredPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     * <p/>
     * <b>Failure scenario:</b>given pageContext is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetRequiredPageContextAttributeNullPageContext() throws Exception {
        try {
            NodeFormatterUtils.getRequiredPageContextAttribute(null, TITLE, String.class);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getRequiredPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     * <p/>
     * <b>Failure scenario:</b>given attributeName is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetRequiredPageContextAttributeNullAttributeName() throws Exception {
        try {
            NodeFormatterUtils.getRequiredPageContextAttribute(pageContext, null, String.class);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getRequiredPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     * <p/>
     * <b>Failure scenario:</b>given attributeName is an empty String
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetRequiredPageContextAttributeEmptyAttributeName() throws Exception {
        try {
            NodeFormatterUtils.getRequiredPageContextAttribute(pageContext, " ", String.class);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getRequiredPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     * <p/>
     * <b>Failure scenario:</b>given expectedType is <tt>null</tt>
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetRequiredPageContextAttributeNullExcpectedType() throws Exception {
        try {
            NodeFormatterUtils.getRequiredPageContextAttribute(pageContext, TITLE, null);
            fail("should throw");
        } catch (IllegalArgumentException e) {
            //expected
        }
    }

    /**
     * This method tests the correct behavior of {@link NodeFormatterUtils#getRequiredPageContextAttribute(
     *javax.servlet.jsp.PageContext,String,Class)}.
     * <p/>
     * <b>Failure scenario:</b>retrieved value has wrong type
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    public void testGetRequiredPageContextAttributeWrongType() throws Exception {
        pageContext.setAttribute(TITLE, new Integer(6));
        try {
            NodeFormatterUtils.getRequiredPageContextAttribute(pageContext, TITLE, String.class);
            fail("should throw");
        } catch (BreadCrumbException e) {
            //expected
        }
    }

    /**
     * This method does the test setup needed.
     *
     * @throws Exception in case some unexpected Exception occurs
     */
    protected void setUp() throws Exception {
        super.setUp();
        pageContext.removeAttribute(TITLE);
    }
}
